import 'dart:io';
import 'dart:async';
import 'package:flutter/material.dart';
import 'package:image_crop/image_crop.dart';
import 'package:image_picker/image_picker.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:senior_design_system/senior_design_system.dart';
import 'package:senior_design_tokens/senior_design_tokens.dart';
import 'package:senior_design_tokens/tokens/senior_spacing.dart';

enum AppBarStyle {
  appbar,
  colorfulHeader,
}

class SeniorImageCropper {
  final String takePhotoLabel;
  final String pickImageLabel;
  final String cropperTitle;
  final String cropImageLabel;
  final String cancelCropLabel;
  final AppBarStyle appBarStyle;
  final Function(File croppedImage) onCroppImage;

  const SeniorImageCropper({
    required this.takePhotoLabel,
    required this.pickImageLabel,
    required this.cropperTitle,
    required this.cropImageLabel,
    required this.cancelCropLabel,
    this.appBarStyle = AppBarStyle.appbar,
    required this.onCroppImage,
  });

  void _loadPicker({
    required ImageSource source,
    required BuildContext context,
  }) async {
    XFile? picked = await ImagePicker().pickImage(source: source);
    if (picked != null) {
      Navigator.pop(context);
      Navigator.push(
        context,
        MaterialPageRoute(
          builder: (context) => CropperView(
            pickedImage: File(picked.path),
            cropperTitle: cropperTitle,
            cropImageLabel: cropImageLabel,
            cancelCropLabel: cancelCropLabel,
            appBarStyle: appBarStyle,
            onCropImage: onCroppImage,
          ),
        ),
      );
    }
  }

  void pickImage(BuildContext context) {
    showModalBottomSheet<void>(
      context: context,
      builder: (BuildContext context) {
        return Container(
          color: SeniorColors.pureWhite,
          child: Padding(
            padding: const EdgeInsets.all(
              SeniorSpacing.normal,
            ),
            child: Column(
              mainAxisAlignment: MainAxisAlignment.center,
              mainAxisSize: MainAxisSize.min,
              children: [
                SeniorButton.icon(
                  label: takePhotoLabel,
                  icon: FontAwesomeIcons.camera,
                  type: SeniorButtonType.primary,
                  onPressed: () {
                    _loadPicker(
                      source: ImageSource.camera,
                      context: context,
                    );
                  },
                ),
                SizedBox(
                  height: SeniorSpacing.medium,
                ),
                SeniorButton.icon(
                  label: pickImageLabel,
                  icon: FontAwesomeIcons.solidFolder,
                  type: SeniorButtonType.secondary,
                  onPressed: () {
                    _loadPicker(
                      source: ImageSource.gallery,
                      context: context,
                    );
                  },
                ),
              ],
            ),
          ),
        );
      },
    );
  }
}

class CropperView extends StatelessWidget {
  final cropKey = GlobalKey<CropState>();
  final File pickedImage;
  final String cropperTitle;
  final String cropImageLabel;
  final String cancelCropLabel;
  final AppBarStyle appBarStyle;
  final Function(File croppedImage) onCropImage;
  final double maximumScale;

  CropperView({
    Key? key,
    required this.pickedImage,
    required this.cropperTitle,
    required this.cropImageLabel,
    required this.cancelCropLabel,
    required this.appBarStyle,
    required this.onCropImage,
    this.maximumScale = 2.0,
  }) : super(key: key);

  Future<void> _cropImage(BuildContext context) async {
    final scale = cropKey.currentState!.scale;
    final area = cropKey.currentState!.area;

    if (area == null) return;

    final sample = await ImageCrop.sampleImage(
      file: pickedImage,
      preferredSize: (2000 / scale).round(),
    );

    final croppedImage = await ImageCrop.cropImage(
      file: sample,
      area: area,
    );

    onCropImage(croppedImage);
    sample.delete();
    Navigator.pop(context);
  }

  void _cancelCrop(BuildContext context) {
    pickedImage.delete();
    Navigator.pop(context);
  }

  AppBar _buildAppBar(BuildContext context) {
    switch(appBarStyle) {
      case AppBarStyle.colorfulHeader:
        return AppBar(
        title: Text(
          cropperTitle,
          style: SeniorTypography.body(
            color: SeniorColors.pureWhite,
          ),
        ),
        backgroundColor: SeniorColors.pureBlack,
        leading: IconButton(
          icon: FaIcon(FontAwesomeIcons.chevronLeft),
          onPressed: () {
            _cancelCrop(context);
          },
          iconSize: SeniorSpacing.normal,
        ),
      );
      case AppBarStyle.appbar:
      default:
        return AppBar(
        title: Text(
          cropperTitle,
          style: SeniorTypography.h4(
            color: SeniorColors.pureWhite,
          ),
        ),
        backgroundColor: SeniorColors.pureBlack,
        leading: IconButton(
          icon: FaIcon(FontAwesomeIcons.chevronLeft),
          onPressed: () {
            _cancelCrop(context);
          },
          iconSize: SeniorSpacing.medium,
        ),
      );
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: _buildAppBar(context),
      body: Container(
        color: SeniorColors.pureBlack,
        child: Column(
          children: <Widget>[
            Expanded(
              child: Padding(
                padding: const EdgeInsets.symmetric(
                  horizontal: SeniorSpacing.medium,
                  vertical: SeniorSpacing.xbig,
                ),
                child: Crop.file(
                  pickedImage,
                  key: cropKey,
                  alwaysShowGrid: true,
                  maximumScale: maximumScale,
                ),
              ),
            ),
            Container(
              decoration: BoxDecoration(
                color: SeniorColors.pureWhite,
              ),
              padding: const EdgeInsets.symmetric(
                vertical: SeniorSpacing.normal,
              ),
              alignment: AlignmentDirectional.center,
              child: Column(
                mainAxisAlignment: MainAxisAlignment.spaceAround,
                children: <Widget>[
                  SeniorButton(
                    label: cropImageLabel,
                    onPressed: () {
                      _cropImage(context);
                    },
                  ),
                  SizedBox(
                    height: SeniorSpacing.small,
                  ),
                  SeniorButton(
                    label: cancelCropLabel,
                    type: SeniorButtonType.ghost,
                    onPressed: () {
                      _cancelCrop(context);
                    },
                  ),
                ],
              ),
            )
          ],
        ),
      ),
    );
  }
}
